@@ -6,7 +6,7 @@ Net::OpenID::URIFetch - fetch and cache content from HTTP URLs
 
 =head1 VERSION
 
-version 1.11
+version 1.12
 
 =head1 DESCRIPTION
 
@@ -24,7 +24,7 @@ isn't much use outside of it. See URI::Fetch for a more general module.
 
 package Net::OpenID::URIFetch;
 BEGIN {
-  $Net::OpenID::URIFetch::VERSION = '1.11';
+  $Net::OpenID::URIFetch::VERSION = '1.12';
 }
 
 use HTTP::Request;
@@ -43,8 +43,19 @@ use constant URI_MOVED_PERMANENTLY => 301;
 use constant URI_NOT_MODIFIED      => 304;
 use constant URI_GONE              => 410;
 
+# Fetch a document, either from cache or from a server
+#    URI -- location of document
+#    CONSUMER -- where to find user-agent and cache
+#    CONTENT_HOOK -- applied to freshly-retrieved document
+#      to normalize it into some particular format/structure
+#    PREFIX -- used as part of the cache key, distinguishes
+#      different content formats and must change whenever
+#      CONTENT_HOOK is switched to a new format; this way,
+#      cache entries from a previous run of this server that
+#      are using a different content format will not kill us.
 sub fetch {
-    my ($class, $uri, $consumer, $content_hook) = @_;
+    my ($class, $uri, $consumer, $content_hook, $prefix) = @_;
+    $prefix ||= '';
 
     if ($uri eq 'x-xrds-location') {
         Carp::confess("Buh?");
@@ -54,10 +65,7 @@ sub fetch {
     my $cache = $consumer->cache;
     my $ref;
 
-    # By prefixing the cache key, we can ensure we won't
-    # get left-over cache items from older versions of Consumer
-    # that used URI::Fetch.
-    my $cache_key = 'URIFetch:'.$uri;
+    my $cache_key = "URIFetch:${prefix}:${uri}";
 
     if ($cache) {
         if (my $blob = $cache->get($cache_key)) {
@@ -114,7 +122,7 @@ sub fetch {
     else {
         my $content = $res->content;
         my $final_uri = $res->request->uri->as_string();
-        my $final_cache_key = "URIFetch:".$final_uri;
+        my $final_cache_key = "URIFetch:${prefix}:${final_uri}";
 
         if ($res->content_encoding && $res->content_encoding eq 'gzip') {
             $content = Compress::Zlib::memGunzip($content);
@@ -155,7 +163,7 @@ sub fetch {
 
 package Net::OpenID::URIFetch::Response;
 BEGIN {
-  $Net::OpenID::URIFetch::Response::VERSION = '1.11';
+  $Net::OpenID::URIFetch::Response::VERSION = '1.12';
 }
 
 sub new {
